import Inventario, { STATUS_INVENTARIO } from "../models/Inventario.js";
import messages, { sendError, sendResponse } from "../utils/mensagens.js";
import { paginateOptions } from "./common.js";
import { jwtDecode } from "jwt-decode";
import { Validator, ValidationFuncs as v } from "../middlewares/validation/validation.js";
import { ACAO, PERM } from "../models/Grupo.js";
import GrupoService from "../services/auth/GrupoService.js";

export default class InventarioController {
    static async pesquisarInventario(req, res) {
        try {
            const pagina = parseInt(req.query.pagina) || 1;
            const token = req.headers.authorization;
            const tokenDecoded = jwtDecode(token);
            const campus = tokenDecoded.campus;

            const {
                responsavel,
                status,
                data_inicial_inicial: data_inicial_inicial,
                data_inicial_final: data_final_inicial,
                data_final_inicial: data_inicial_final,
                data_final_final: data_final_final,
            } = req.query;

            const filtros = { campus };
            let sort = { sigla: 1 };

            if (responsavel) filtros.responsavel = responsavel;

            if (status) filtros.status = status;

            if (data_inicial_inicial || data_final_inicial) {
                filtros.data_inicio = {};
                if (data_inicial_inicial) filtros.data_inicio.$gte = new Date(data_inicial_inicial);
                if (data_final_inicial) {
                    const dataFinal = new Date(data_final_inicial);
                    dataFinal.setDate(dataFinal.getDate() + 1); // Adiciona 1 dia para incluir a data final
                    filtros.data_inicio.$lte = dataFinal;
                }

                sort = { data_inicio: -1, _id: -1 }; // Se não é único precisa ter um segundo campo único para ordenar

            }

            if (data_inicial_final || data_final_final) {
                filtros.data_final = {};
                if (data_inicial_final) filtros.data_final.$gte = new Date(data_inicial_final);
                if (data_final_final) {
                    const dataFinal = new Date(data_final_final);
                    dataFinal.setDate(dataFinal.getDate() + 1); // Adiciona 1 dia para incluir a data final
                    filtros.data_final.$lte = dataFinal;
                }

                sort = { data_final: -1, _id: -1 }; // Se não é único precisa ter um segundo campo único para ordenar

            }

            const inventarios = await Inventario.paginate(
                { campus: { $elemMatch: { _id: campus } }, ...filtros },
                {
                    ...paginateOptions, ...{
                        sort: sort,
                        page: pagina,
                    }
                }
            );

            inventarios.code = 200;
            inventarios.error = false;
            inventarios.errors = [];

            res.status(200).json({ ...inventarios, error: false, code: 200, message: messages.httpCodes[200], errors: [] });
        } catch (error) {
            return sendError(res, 500, messages.httpCodes[500]);
        }
    }

    static async inventarioAtual(req, res) {

        const token = req.headers.authorization;
        const tokenDecoded = jwtDecode(token);
        const campus = tokenDecoded.campus;

        const inventario = await Inventario.findOne({ status: "Em andamento", campus: campus })
            .sort({ data_fim: -1 })
            .populate('responsavel', 'nome')
            .exec();

        if (!inventario) {
            return sendError(res, 404, "Não ha um inventário em andamento!");
        }

        return sendResponse(res, 200, { data: inventario });
    }

    static async listarInventarioID(req, res) {
        try {
            let val = new Validator(req.params);
            await val.validate("id", v.required(), v.mongooseID());
            if (val.anyErrors()) return sendError(res, 400, val.getErrors());

            // DEFINIR OQ O MOBILE NECESSITA E SOMENTE PUXAR ISSO PLEASE
            const inventario = await Inventario.findById(req.params.id);

            if (!inventario) {
                return sendError(res, 404);
            }

            return sendResponse(res, 200, { data: inventario });
        } catch (err) {
            return sendError(res, 500, messages.httpCodes[500]);
        }
    }

    static async criarInventario(req, res) {

        req.body.responsavel = req.decodedToken.id;
        req.body.campus = req.decodedToken.campus;
        req.body.status = STATUS_INVENTARIO.ANDAMENTO;

        const inventario = await Inventario.create({ ...req.body });

        return sendResponse(res, 201, {
            data: inventario
        });
    }

    static atualizarInventario = async (req, res) => {

        const inventario = req.validateResult.inventario;

        let permissaoRecurso = GrupoService.possuiPermissaoRecurso(req, "reserva", inventario.campus, inventario.responsavel, PERM.INVENTARIO, ACAO.EDITAR);
        if (permissaoRecurso !== true) {
            return sendError(res, 403, "Você não tem permissão para alterar este inventário!");
        }

        // Só atualiza os campos que foram enviados
        for (let key in req.body) {
            inventario[key] = req.body[key];
        }

        await inventario.save();
        return sendResponse(res, 200, {
            data: inventario
        });
    };

    static async deletarInventario(req, res) {

        const inventario = req.validateResult.inventario;

        let permissaoRecurso = GrupoService.possuiPermissaoRecurso(req, "reserva", inventario.campus, inventario.responsavel, PERM.INVENTARIO, ACAO.DELETAR);
        if (permissaoRecurso !== true) {
            return sendError(res, 403, "Você não tem permissão para deletar este inventário!");
        }

        await Inventario.deleteOne({ _id: inventario._id });
        return sendResponse(res, 200);
    }
}