import Item from "../models/Item.js";
import Campus from "../models/Campus.js";
import Setor, { setorPopulateSelect } from "../models/Setor.js";
import GrupoService from "../services/auth/GrupoService.js";
import messages, { sendError, sendResponse } from "../utils/mensagens.js";
import { paginateOptions } from "./common.js";
import { ACAO, PERM } from "../models/Grupo.js";
import Inventario from "../models/Inventario.js";
import ItemSemEtiqueta from "../models/itemSemEtiqueta.js";
import { usuarioPopulateSelect } from "../models/Usuario.js";
import { jwtDecode } from "jwt-decode";

export default class ItemController {
    static async pesquisarItem(req, res) {
        const pagina = parseInt(req.query.pagina) || 1;

        // Atributos esperados na requisição, validação foi feita no middleware antes de chegar aqui
        const {
            etiqueta,
            nao_tiquetado,
            encontrado,
            nome,
            estado,
            ativo,
            ocioso,
            inventario,
            setor,
            auditor,
            responsavel
        } = req.query;

        const filtros = {};

        if (etiqueta) filtros.etiqueta = etiqueta;
        if (nao_tiquetado) filtros.nao_tiquetado = nao_tiquetado;
        if (encontrado) filtros.encontrado = encontrado;
        if (nome) filtros.nome = { $regex: new RegExp(nome, "i") };
        if (estado) filtros.estado = estado;
        if (ativo) filtros.ativo = ativo;
        if (ocioso) filtros.ocioso = ocioso;
        if (inventario) filtros.inventario = inventario;
        if (setor) filtros.setor = setor;
        if (auditor) filtros.auditor = auditor;
        if (responsavel) filtros.responsavel = responsavel;

        const itens = await Item.paginate(
            { ...filtros },
            {
                ...paginateOptions, ...{
                    sort: { sigla: 1 },
                    page: pagina,
                },
            });

        itens.code = 200;
        itens.error = false;
        itens.errors = [];

        return sendResponse(res, 200, itens);
    }

    static async listarUltimosItens(req, res) {

        const pagina = parseInt(req.query.pagina) || 1;
        const token = req.headers.authorization;
        const tokenDecoded = jwtDecode(token);
        const campus = tokenDecoded.campus;

        const inventario = await Inventario.findOne({ status: "Finalizada", campus: campus })
            .sort({ data_fim: -1 })
            .exec();

        // Atributos esperados na requisição, validação foi feita no middleware antes de chegar aqui
        const {
            setor,
            etiqueta
        } = req.query;
        
        const filtros = { inventario };

        if (inventario) filtros.inventario = inventario;
        if (setor) filtros.setor = setor;
        if (etiqueta) filtros.etiqueta = etiqueta;

        const itens = await Item.paginate(
            { ...filtros },
            {
                ...paginateOptions, ...{
                    sort: { sigla: 1 },
                    page: pagina,
                    populate: [
                        { path: "setor", select: setorPopulateSelect },
                        { path: "auditor", select: usuarioPopulateSelect },
                        { path: "responsavel", select: usuarioPopulateSelect}
                    ],
                },
            });


        if (Array.isArray(itens.data) && itens.data.length === 0) {
            itens.code = 404;
            itens.error = true;
            itens.errors = ["Item não encontrada!"];
            return sendResponse(res, 404, itens);
        }

        itens.code = 200;
        itens.error = false;
        itens.errors = [];

        return sendResponse(res, 200, itens);
    }

    static async listarItensInventarioAtual(req, res) {
        const pagina = parseInt(req.query.pagina) || 1;
        const token = req.headers.authorization;
        const tokenDecoded = jwtDecode(token);
        const campus = tokenDecoded.campus;
    
        const inventario = await Inventario.findOne({ status: "Em andamento", campus: campus })
            .sort({ data_fim: -1 })
            .exec();
    
        const { setor, etiqueta } = req.query;
        const filtros = { inventario };
        
        if (inventario) filtros.inventario = inventario;
        if (setor) filtros.setor = setor;
        if (etiqueta) filtros.etiqueta = etiqueta;

        const itensComEtiqueta = await Item.paginate(
            { ...filtros },
            {
                ...paginateOptions, ...{
                    sort: { sigla: 1 },
                    limit: 200,
                    page: pagina,
                    populate: [
                        { path: "setor", select: setorPopulateSelect },
                        { path: "auditor", select: usuarioPopulateSelect },
                        { path: "responsavel", select: usuarioPopulateSelect }
                    ],
                },
            });
    
        const itensSemEtiqueta = await ItemSemEtiqueta.paginate(
            { ...filtros },
            {
                ...paginateOptions, ...{
                    sort: { sigla: 1 },
                    page: pagina,
                    limit: 200,
                    populate: [
                        { path: "setor", select: setorPopulateSelect },
                        { path: "auditor", select: usuarioPopulateSelect }
                    ],
                },
            });
        
        const todosItens = [...itensComEtiqueta.data, ...itensSemEtiqueta.data];
    
        const totalItens = itensComEtiqueta.resultados + itensSemEtiqueta.resultados;
    
        const totalPaginas = Math.ceil(totalItens / itensComEtiqueta.limite); 

        const resposta = {
            data: todosItens,         
            resultados: totalItens,     
            limite: itensComEtiqueta.limite,  
            totalPaginas: totalPaginas,  
            pagina: pagina
        };
    
        return sendResponse(res, 200, resposta);
    }
    
    
    static async listarItemID(req, res) {

        const { id } = req.params;
        const item = await Item.findById(id);

        if (!item) {
            return res.status(404).json({ data: [], error: true, code: 404, message: messages.httpCodes[404], errors: ["Item não encontrada!"] });
        }

        res.status(200).json({ item, error: false, code: 200, message: messages.httpCodes[200], errors: [] });

    }

    static async criarItem(req, res) {
        try {
            const setor = await Setor.findById(req.body.setor);
    
            if (!setor) {
                return sendError(res, 404, "Setor não encontrado!");
            }
    
            const campus = await Campus.findById(setor.campus);
    
            if (!campus) {
                return sendError(res, 404, "Campus não encontrado!");
            }
    
            let permissaoRecurso = GrupoService.possuiPermissaoRecurso(req, "item", campus, false, PERM.ITEM, ACAO.CRIAR);
            if (permissaoRecurso !== true) {
                return sendError(res, 403, "Você não tem permissão para criar este item!");
            }
    
            const item = await Item.create(req.body);
    
            const populatedItem = await Item.findById(item._id)
                .populate([
                    { path: "setor", select: "nome" }, 
                    { path: "auditor", select: "nome email" },
                    { path: "responsavel", select: "nome email" } 
                ]);
    
            return sendResponse(res, 201, {
                data: populatedItem
            });
    
        } catch (error) {
            // Lidar com erros
            return sendError(res, 500, "Ocorreu um erro ao criar o item.");
        }
    }

    static async atualizarItem(req, res) {

        const { id } = req.params;
        let item = await Item.findById(id);

        if (!item) {
            return res.status(404).json({ data: [], error: true, code: 404, message: messages.httpCodes[404], errors: ["Item não encontrado!"] });
        }

        item = await Item.findByIdAndUpdate(id, req.body, { new: true });

        return sendResponse(res, 200, {
            data: item
        });
    }

    static async deletarItem(req, res) {
        try {
            const { id } = req.params;
            const item = await Item.findById(id);

            const itemSemEtiqueta = await ItemSemEtiqueta.findById(id);

            if (!item && !itemSemEtiqueta) {
                return res.status(404).json({ data: [], error: true, code: 404, message: messages.httpCodes[404], errors: ["Item não encontrado!"] });
            }

            if (item) {
                await Item.findByIdAndDelete(id);
            }

            if (itemSemEtiqueta) {
                 await ItemSemEtiqueta.findByIdAndDelete(id);
            }

            res.status(200).json({ data: item || itemSemEtiqueta, error: false, code: 200, message: messages.httpCodes[200], errors: [] });
        } catch (err) {
            return res.status(500).json({ data: [], error: true, code: 500, message: messages.httpCodes[500], errors: ["Servidor encontrou um erro interno."] });
        }
    }

    static async criarItemSemEtiqueta(req, res) {
        const setor = req.body.setor;
        const campus = setor.campus;

        const item = await ItemSemEtiqueta.create({ ...req.body, campus, setor });

        return sendResponse(res, 201,{ data: [item]});
    }
}